"""
Created on 19.02.2023

@author: wf
"""

from urllib.request import urlopen

import xmltodict
from sidif.sidif import DataInterchange, SiDIFParser

from meta.uml import PlantUml
from tests.basetest import Basetest


class TestPlantUml(Basetest):
    """
    test plantuml handling
    """

    def setUp(self, debug=False, profile=True):
        Basetest.setUp(self, debug=debug, profile=profile)

    def testIssue4(self):
        """
        https://github.com/WolfgangFahl/py-sidif/issues/4
        convert dict tree to sidif
        """
        uri = "https://raw.githubusercontent.com/semstats/semstats.github.io/master/2019/ceur/workshop.xml"
        xml = urlopen(uri).read().decode()
        wsdict = xmltodict.parse(xml)
        if self.debug:
            print(wsdict)
        dif = DataInterchange.ofDict(wsdict, context="ceurws")
        sidifStr = dif.asSiDIF()
        if self.debug:
            print(sidifStr)
        self.assertTrue("workshop isA Topic" in sidifStr)
        self.assertTrue("homepage isA Property" in sidifStr)
        uml = PlantUml(
            title="CEUR-WS", copyRight="© Christoph Lange and contributors 2012–2020"
        )
        uml.fromDIF(dif)
        debug = self.debug
        #debug = True
        if debug:
            print(uml)
        self.assertTrue("""workshop " 1" -- "editors *" editor""" in uml.uml)

    def getDif(self, url, title: str="Presentation"):
        """
        """
        sp = SiDIFParser(debug=self.debug)
        parsed, error = sp.parseUrl(url, title=title)
        self.assertTrue(error is None)
        # self.debug=True
        if self.debug:
            sp.printResult(parsed)
        dif = parsed[0]
        return dif

    def getPlantUmlFromSidifUrl(self, url, title, copy_right, debug: bool = None):
        if debug is None:
            debug = self.debug
        dif = self.getDif(url, title)
        uml = PlantUml(title=title, copyRight=copy_right)
        uml.fromDIF(dif)
        if debug:
            print(uml)
        return uml

    def testIssue5(self):
        """
        https://github.com/WolfgangFahl/py-sidif/issues/5
        convert sidif to plantuml #5
        """
        self.baseUrl = "https://raw.githubusercontent.com/BITPlan/org.sidif.triplestore/master/src/test/resources/sidif"
        url = f"{self.baseUrl}/presentation.sidif"
        title = "Presentation"
        copy_right = "© BITPlan GmbH 2015-2024"
        uml = self.getPlantUmlFromSidifUrl(url, title, copy_right)
        self.assertTrue("package Presentation {" in uml.uml)
        self.assertTrue("class Icon {" in uml.uml)
        self.assertTrue("author:Text" in uml.uml)

    def testIssue25(self):
        """
        https://github.com/WolfgangFahl/pyMetaModel/issues/25
        support inheritance of topics

        """
        url = "https://raw.githubusercontent.com/WolfgangFahl/pyMetaModel/main/examples/infrastructure/infrastructure.sidif"
        title = "Infrastructure"
        copy_right = "© BITPlan GmbH 2024"
        uml = self.getPlantUmlFromSidifUrl(url, title, copy_right, debug=True)
        self.assertTrue("Harddisk extends Device" in uml.uml)



"""
Created on 2024-07-29

@author: wf
"""

import tempfile
from pathlib import Path
from tests.basetest import Basetest
from meta.metamodel_cmd import MetaModelCmd
import subprocess

class TestGenerators(Basetest):
    """
    test generators
    """

    def setUp(self, debug=False, profile=True):
        Basetest.setUp(self, debug=debug, profile=profile)
        self.temp_dir = tempfile.TemporaryDirectory()
        self.examples_dir = Path(__file__).parent.parent / "examples"
        self.mm_cmd = MetaModelCmd()
        self.sidif_files = list(self.examples_dir.glob("**/*.sidif"))

    def tearDown(self):
        self.temp_dir.cleanup()
        super().tearDown()


    def generate_plantuml(self,sidif_file,withAt:bool=False):
        args = ["-i", str(sidif_file), "-u"]
        if self.debug:
            args.append("-d")
        if withAt:
            args.append("--withAt")
        uml = self.mm_cmd.genUml(self.mm_cmd.getArgParser().parse_args(args))
        self.assertIsNotNone(uml)
        self.assertTrue(len(uml.uml) > 0)
        if self.debug:
            print(uml)
            pass
        return uml


    def run_command(self, command, *args):
        """Run a command and return its output"""
        result = subprocess.run([command, *args], capture_output=True, text=True)
        self.assertEqual(result.returncode, 0)
        return result.stdout

    def generate_yaml(self,sidif_file)->str:
        """Generate YAML from SIDIF file"""
        args = ["-i", str(sidif_file), "-l"]
        yaml = self.mm_cmd.genLinkML(self.mm_cmd.getArgParser().parse_args(args))
        return yaml

    def test_plantuml_generation(self):
        """
        test plant uml generation
        """
        for sidif_file in self.sidif_files:
            uml=self.generate_plantuml(sidif_file, withAt=False)
            uml_with_at = self.generate_plantuml(sidif_file, withAt=True)
            self.assertTrue(str(uml_with_at).startswith("@startuml"))
            self.assertTrue(str(uml_with_at).endswith("@enduml"))

    def test_linkml_generation(self):
        for sidif_file in self.sidif_files:
            yaml=self.generate_yaml(sidif_file)
            if self.debug:
                print(yaml)
        self.assertTrue(len(yaml) > 0)



"""
Created on 2022-11-23

@author: wf
"""

import datetime
from dataclasses import dataclass

import mwparserfromhell
from wikibot3rd.smw import SMWClient
from wikibot3rd.wikiclient import WikiClient
from wikibot3rd.wikiuser import WikiUser


@dataclass
class MediaWikiContext:
    """Class for keeping track of MediaWiki Context"""

    wikiId: str
    wiki_url: str
    context: str
    since: datetime.datetime
    master: str

    def sidif_url(self):
        """
        return the sidif url
        """
        url = f"{self.wiki_url}/index.php/{self.context}#sidif"
        return url

    def read_sidif(self) -> str:
        """

        Read the SiDIF for this Mediawiki context

        Returns:
            str: the SiDIF
        """
        sidif = None
        wikiusers = WikiUser.getWikiUsers(lenient=True)
        if not self.wikiId in wikiusers:
            raise ValueError(f"unknown wiki {self.wikiId}")
        else:
            wikiUser = wikiusers[self.wikiId]
            self.wikiClient = WikiClient.ofWikiId(wikiUser.wikiId)
            if self.wikiClient.needsLogin():
                self.wikiClient.login()
            pageTitle = f"{self.context}"
            page = self.wikiClient.getPage(pageTitle)
            if page.exists:
                markup = page.text()
                mw_code = mwparserfromhell.parse(markup)
                sidif_sections = mw_code.get_sections(matches="sidif")
                if len(sidif_sections) != 1:
                    raise Exception(
                        f"found {len(sidif_sections)} sidif sections in {self.context}@{wikiUser.wikiId} but expected exactly 1"
                    )
                for node in sidif_sections[0].filter_tags(matches="source"):
                    sidif = str(node.contents)
        return sidif


class SMWAccess:
    """
    access to semantic MediaWiki
    """

    def __init__(self, wikiId: str = "wiki", debug: bool = False):
        """
        constructor
        """
        self.wikiId = wikiId
        self.smw = self.getSMW(wikiId)
        self.debug = debug
        self.url = f"{self.wikiClient.wikiUser.getWikiUrl()}"

    def getSMW(self, wikiId: str):
        """
        get the semantic mediawiki access
        """
        self.wikiClient = WikiClient.ofWikiId(wikiId)
        if self.wikiClient.needsLogin():
            self.wikiClient.login()
        smw = SMWClient(self.wikiClient.getSite())
        return smw

    def getMwContexts(self):
        """
        get the contexts
        """
        ask = """{{#ask: [[Concept:Context]]
|mainlabel=Context
| ?Context name = name
| ?Context since = since
| ?Context master = master

|sort=Context name
|order=ascending
}}"""
        mw_contexts = {}
        context_records = self.smw.query(ask, "list of contexts")
        for context_name, context_record in context_records.items():
            if self.debug:
                print(context_record)
            mw_contexts[context_name] = MediaWikiContext(
                self.wikiId,
                self.url,
                context_name,
                context_record["since"],
                context_record["master"],
            )
        return mw_contexts



"""
Created on 2023-02-25

@author: wf
"""

from meta.smw_type import SMW_Type
from tests.basemwtest import BaseMediawikiTest


class TestMetaModel(BaseMediawikiTest):
    """
    test round trip meta model handling
    """

    def setUp(self, debug=False, profile=True):
        """
        setUp
        """
        BaseMediawikiTest.setUp(self, debug=debug, profile=profile)
        for wikiId in ["wiki"]:
            self.getSMWAccess(wikiId, save=True)

    def testSMW_Type(self):
        """
        test SMW_Type handling
        """
        debug = self.debug
        smwAccess = self.smwAccessMap["wiki"]
        ask_query = SMW_Type.askQuery()
        did = smwAccess.smw.query(ask_query)
        for record in did.values():
            smw_type = SMW_Type.from_dict(record) # @UndefinedVariable
            if debug:
                print(smw_type)
            self.assertIsInstance(smw_type, SMW_Type)


from meta.metamodel import Property, Topic
from tests.basetest import Basetest


class TestTopic(Basetest):
    """
    test Topic methods - see https://github.com/WolfgangFahl/pyMetaModel/issues/36
    """

    def test_get_primary_key_property(self):
        """test get_primary_key_property returns the primaryKey property or None"""
        topic = Topic()
        topic.properties = {
            "name": Property(name="name", primaryKey=True),
            "at": Property(name="at"),
        }
        pk = topic.get_primary_key_property()
        self.assertIsNotNone(pk)
        self.assertEqual(pk.name, "name")

    def test_get_primary_key_property_none(self):
        """test get_primary_key_property returns None when no primaryKey exists"""
        topic = Topic()
        topic.properties = {"at": Property(name="at")}
        self.assertIsNone(topic.get_primary_key_property())
